#include <Adafruit_NeoPixel.h>
#include <ESP8266WiFi.h>
#include <EEPROM.h>

const char* ssid        = "";
const char* password    = "";

#define ANIMATION_COUNT 9
#define LED_COUNT 9
#define LED_PIN   D5

bool ledArray[LED_COUNT];
bool animation[ANIMATION_COUNT];
WiFiServer server(80);
int delayCounter;
uint addr = 0;
//status data
struct {
  uint val = 0;
  bool leds[LED_COUNT];
  bool animations[ANIMATION_COUNT];
} data;

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

void  updateEEPROM(bool isanimation) {
  if(!isanimation) {
    clearAnimations();
  }
  memcpy(data.leds, ledArray, sizeof(data.leds[0])*LED_COUNT);
  memcpy(data.animations, animation, sizeof(data.animations[0])*ANIMATION_COUNT);
  EEPROM.put(addr, data);
  EEPROM.commit();
}

void setup() {
  Serial.begin(115200);
  delay(10);

  EEPROM.begin(512);
  //get values from EEPROM
  EEPROM.get(addr, data);
  memcpy(ledArray, data.leds, sizeof(data.leds[0])*LED_COUNT);
  memcpy(animation, data.animations, sizeof(data.animations[0])*ANIMATION_COUNT);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.print("Use this URL to connect: ");
  Serial.print("http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");

  for(int led; led < LED_COUNT; led++) {
    strip.setPixelColor(led, 0,0,0);
  }

  delayCounter = 0;

  strip.begin();
  strip.show();
}

void clearAnimations() {
  animation[0] = false;
  animation[1] = false;
}


void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    delayCounter++;
    if(delayCounter >= 50) {
      Serial.print("*********** ");
      Serial.println(animation[1]);
      delayCounter = 0;
      if(animation[0]) {
        theaterChaseRainbow(50);
      }
      else if(animation[1]) {
        rainbowCycle(20);
      }


      else {
        for(int led = 0; led < LED_COUNT; led++) {
          if(ledArray[led]) {
            strip.setPixelColor(led, 50, 135 + random(120), 0);
          }
          else{
            strip.setPixelColor(led, 0,0,0);
          }
        }
      }
    }

    strip.show();
    return;
  }

  // Wait until the client sends some data
  Serial.println("new client");
  while(!client.available()){
    delay(1);
  }

  // Read the first line of the request
  String request = client.readStringUntil('\r');
  Serial.println(request);
  client.flush();

  // Match the request

  int value = LOW;
  if (request.indexOf("/shamash=ON") != -1)  {
    ledArray[4] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/shamash=OFF") != -1)  {
    ledArray[4] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/firstnight=ON") != -1)  {
    ledArray[0] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/firstnight=OFF") != -1)  {
    ledArray[0] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/secondnight=ON") != -1)  {
    ledArray[1] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/secondnight=OFF") != -1)  {
    ledArray[1] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/thirdnight=ON") != -1)  {
    ledArray[2] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/thirdnight=OFF") != -1)  {
    ledArray[2] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/fourthnight=ON") != -1)  {
    ledArray[3] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/fourthnight=OFF") != -1)  {
    ledArray[3] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/fifthnight=ON") != -1)  {
    ledArray[5] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/fifthnight=OFF") != -1)  {
    ledArray[5] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/sixthnight=ON") != -1)  {
    ledArray[6] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/sixthnight=OFF") != -1)  {
    ledArray[6] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/seventhnight=ON") != -1)  {
    ledArray[7] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/seventhnight=OFF") != -1)  {
    ledArray[7] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/eighthnight=ON") != -1)  {
    ledArray[8] = true;
    updateEEPROM(false);
  }
  if (request.indexOf("/eighthnight=OFF") != -1)  {
    ledArray[8] = false;
    updateEEPROM(false);
  }

  if (request.indexOf("/Animation_1=ON") != -1)  {
    animation[0] = true;
    updateEEPROM(true);
  }

  if (request.indexOf("/Animation_2=ON") != -1)  {
    animation[1] = true;
    updateEEPROM(true);
  }



// Set ledPin according to the request
//digitalWrite(ledPin, value);

  // Return the response
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println(""); //  do not forget this one
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");

  client.println("<br><br>");
  client.println("<a href=\"/shamash=ON\"\"><button>Turn Shamash Candle On </button></a>");
  client.println("<a href=\"/shamash=OFF\"\"><button>Turn Shamash Candle Off </button></a><br />");
  client.println("<a href=\"/firstnight=ON\"\"><button>Turn First Night Candle On </button></a>");
  client.println("<a href=\"/firstnight=OFF\"\"><button>Turn First Night Candle Off </button></a><br />");
  client.println("<a href=\"/secondnight=ON\"\"><button>Turn Second Night Candle On </button></a>");
  client.println("<a href=\"/secondnight=OFF\"\"><button>Turn Second Night Candle Off </button></a><br />");
  client.println("<a href=\"/thirdnight=ON\"\"><button>Turn Third Night Candle On </button></a>");
  client.println("<a href=\"/thirdnight=OFF\"\"><button>Turn Third Night Candle Off </button></a><br />");
  client.println("<a href=\"/fourthnight=ON\"\"><button>Turn Fourth Night Candle On </button></a>");
  client.println("<a href=\"/fourthnight=OFF\"\"><button>Turn Fourth Night Candle Off </button></a><br />");
  client.println("<a href=\"/fifthnight=ON\"\"><button>Turn Fifth Night Candle On </button></a>");
  client.println("<a href=\"/fifthnight=OFF\"\"><button>Turn Fifth Night Candle Off </button></a><br />");
  client.println("<a href=\"/sixthnight=ON\"\"><button>Turn Sixth Night Candle On </button></a>");
  client.println("<a href=\"/sixthnight=OFF\"\"><button>Turn Sixth Night Candle Off </button></a><br />");
  client.println("<a href=\"/seventhnight=ON\"\"><button>Turn Seventh Night Candle On </button></a>");
  client.println("<a href=\"/seventhnight=OFF\"\"><button>Turn Seventh Night Candle Off </button></a><br />");
  client.println("<a href=\"/eighthnight=ON\"\"><button>Turn Eighth Night Candle On </button></a>");
  client.println("<a href=\"/eighthnight=OFF\"\"><button>Turn Eighth Night Candle Off </button></a><br />");
  client.println("<br /><br /><br /><br />");
  client.println("<a href=\"/Animation_1=ON\"\"><button>Turn Theater Animation On </button></a>");
  client.println("<a href=\"/Animation_2=ON\"\"><button>Turn Rainbow Cycle On </button></a>");
  client.println("</html>");

  delay(1);
  Serial.println("Client disonnected");
  Serial.println("");
}

void theaterChaseRainbow(int SpeedDelay) {
  byte *c;

  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
        for (int i=0; i < LED_COUNT; i=i+3) {
          c = Wheel( (i+j) % 255);
          strip.setPixelColor(i+q, *c, *(c+1), *(c+2));    //turn every third pixel on
        }
        strip.show();

        delay(SpeedDelay);

        for (int i=0; i < LED_COUNT; i=i+3) {
          strip.setPixelColor(i+q, 0,0,0);        //turn every third pixel off
        }
    }
  }
}

byte * Wheel(byte WheelPos) {
  static byte c[3];

  if(WheelPos < 85) {
   c[0]=WheelPos * 3;
   c[1]=255 - WheelPos * 3;
   c[2]=0;
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   c[0]=255 - WheelPos * 3;
   c[1]=0;
   c[2]=WheelPos * 3;
  } else {
   WheelPos -= 170;
   c[0]=0;
   c[1]=WheelPos * 3;
   c[2]=255 - WheelPos * 3;
  }

  return c;
}

void fadeToBlack(int ledNo, byte fadeValue) {
 #ifdef ADAFRUIT_NEOPIXEL_H
    // NeoPixel
    uint32_t oldColor;
    uint8_t r, g, b;
    int value;

    oldColor = strip.getPixelColor(ledNo);
    r = (oldColor & 0x00ff0000UL) >> 16;
    g = (oldColor & 0x0000ff00UL) >> 8;
    b = (oldColor & 0x000000ffUL);

    r=(r<=10)? 0 : (int) r-(r*fadeValue/256);
    g=(g<=10)? 0 : (int) g-(g*fadeValue/256);
    b=(b<=10)? 0 : (int) b-(b*fadeValue/256);

    strip.setPixelColor(ledNo, r,g,b);
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H
   // FastLED
   leds[ledNo].fadeToBlackBy( fadeValue );
 #endif
}

void rainbowCycle(int SpeedDelay) {
  byte *c;
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< LED_COUNT; i++) {
      c=Wheel(((i * 256 / LED_COUNT) + j) & 255);
      strip.setPixelColor(i, *c, *(c+1), *(c+2));
    }
    strip.show();
    delay(SpeedDelay);
  }
}
